@@ -1,6 +1,6 @@
 /* ============================================================
 * QupZilla - WebKit based browser
-* Copyright (C) 2010-2013  David Rosca <nowrep@gmail.com>
+* Copyright (C) 2010-2014  David Rosca <nowrep@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 * ============================================================ */
 #include "networkmanager.h"
-#include "qupzilla.h"
+#include "browserwindow.h"
 #include "autofill.h"
 #include "networkmanagerproxy.h"
 #include "mainapplication.h"
@@ -27,9 +27,11 @@
 #include "networkproxyfactory.h"
 #include "certificateinfowidget.h"
 #include "qztools.h"
+#include "qzsettings.h"
 #include "acceptlanguage.h"
 #include "cabundleupdater.h"
 #include "settings.h"
+#include "datapaths.h"
 #include "passwordmanager.h"
 #include "sslerrordialog.h"
 #include "schemehandlers/adblockschemehandler.h"
@@ -37,20 +39,20 @@
 #include "schemehandlers/fileschemehandler.h"
 #include "schemehandlers/ftpschemehandler.h"
 
+#include <QDir>
 #include <QFormLayout>
 #include <QLabel>
 #include <QLineEdit>
 #include <QCheckBox>
 #include <QDialogButtonBox>
 #include <QNetworkDiskCache>
-#include <QDir>
+#include <QSslCipher>
 #include <QSslSocket>
 #include <QSslConfiguration>
 #include <QDateTime>
 #include <QMessageBox>
 #include <QAuthenticator>
 #include <QDirIterator>
-#include <QDebug>
 
 static QString fileNameForCert(const QSslCertificate &cert)
 {
@@ -70,6 +72,8 @@ NetworkManager::NetworkManager(QObject* parent)
     : NetworkManagerProxy(parent)
     , m_adblockManager(0)
     , m_ignoreAllWarnings(false)
+    , m_disableWeakCiphers(true)
+    , m_disableSSLv3(true)
 {
     connect(this, SIGNAL(authenticationRequired(QNetworkReply*,QAuthenticator*)), this, SLOT(authentication(QNetworkReply*,QAuthenticator*)));
     connect(this, SIGNAL(proxyAuthenticationRequired(QNetworkProxy,QAuthenticator*)), this, SLOT(proxyAuthentication(QNetworkProxy,QAuthenticator*)));
@@ -83,6 +87,7 @@ NetworkManager::NetworkManager(QObject* parent)
 
     m_proxyFactory = new NetworkProxyFactory();
     setProxyFactory(m_proxyFactory);
+
     loadSettings();
 }
 
@@ -90,7 +95,7 @@ void NetworkManager::loadSettings()
 {
     Settings settings;
 
-    if (settings.value("Web-Browser-Settings/AllowLocalCache", true).toBool() && !mApp->isPrivateSession()) {
+    if (settings.value("Web-Browser-Settings/AllowLocalCache", true).toBool() && !mApp->isPrivate()) {
         QNetworkDiskCache* cache = mApp->networkCache();
         cache->setMaximumCacheSize(settings.value("MaximumCacheSize", 50).toInt() * 1024 * 1024); //MegaBytes
         setCache(cache);
@@ -100,27 +105,17 @@ void NetworkManager::loadSettings()
     m_doNotTrack = settings.value("DoNotTrack", false).toBool();
     m_sendReferer = settings.value("SendReferer", true).toBool();
     settings.endGroup();
+
     m_acceptLanguage = AcceptLanguage::generateHeader(settings.value("Language/acceptLanguage", AcceptLanguage::defaultLanguage()).toStringList());
 
-    // Falling back to Qt 4.7 default behavior, use SslV3 by default
-    // Fixes issue with some older servers closing the connection
-
-    // However, it also makes some servers requesting TLS ClientHello
-    // not working, or showing invalid certificates.
-    // See #921
-
-    // QSslConfiguration config = QSslConfiguration::defaultConfiguration();
-    // config.setProtocol(QSsl::SslV3);
-    // QSslConfiguration::setDefaultConfiguration(config);
-
 #if defined(Q_OS_WIN) || defined(Q_OS_HAIKU) || defined(Q_OS_OS2)
-    QString certDir = mApp->PROFILEDIR + "certificates";
+    QString certDir = DataPaths::currentProfilePath() + "/certificates";
     QString bundlePath = certDir + "/ca-bundle.crt";
     QString bundleVersionPath = certDir + "/bundle_version";
 
     if (!QDir(certDir).exists()) {
-        QDir dir(mApp->PROFILEDIR);
-        dir.mkdir("certificates");
+        QDir dir;
+        dir.mkdir(certDir);
     }
 
     if (!QFile::exists(bundlePath)) {
@@ -162,6 +157,47 @@ void NetworkManager::setSSLConfiguration(QNetworkReply* reply)
     }
 }
 
+void NetworkManager::disableWeakCiphers(bool disable)
+{
+    if (disable) {
+        QStringList blacklist;
+        blacklist << QSL("SRP-AES-256-CBC-SHA")  // open to MitM
+                  << QSL("SRP-AES-128-CBC-SHA"); // open to MitM
+
+        // Disable blacklisted ciphers and ciphers with less than 128b key
+        QList<QSslCipher> acceptedCiphers;
+        foreach (const QSslCipher &c, QSslSocket::defaultCiphers()) {
+            if (!blacklist.contains(c.name()) && c.usedBits() >= 128)
+                acceptedCiphers.append(c);
+        }
+        QSslSocket::setDefaultCiphers(acceptedCiphers);
+    }
+    else {
+        QSslSocket::setDefaultCiphers(QSslSocket::supportedCiphers());
+    }
+}
+
+void NetworkManager::disableSSLv3(bool disable)
+{
+    QSsl::SslProtocol tlsProtocol;
+    QSsl::SslProtocol defaultProtocol;
+
+#if QT_VERSION < QT_VERSION_CHECK(4,8,0)
+    tlsProtocol = QSsl::TlsV1;
+    defaultProtocol = QSsl::SslV3;
+#elif QT_VERSION < QT_VERSION_CHECK(5,0,0)
+    tlsProtocol = QSsl::TlsV1;
+    defaultProtocol = QSsl::TlsV1SslV3;
+#else
+    tlsProtocol = QSsl::TlsV1_0;
+    defaultProtocol = QSsl::TlsV1SslV3;
+#endif
+
+    QSslConfiguration config = QSslConfiguration::defaultConfiguration();
+    config.setProtocol(disable ? tlsProtocol : defaultProtocol);
+    QSslConfiguration::setDefaultConfiguration(config);
+}
+
 static inline uint qHash(const QSslCertificate &cert)
 {
     return qHash(cert.toPem());
@@ -339,7 +375,7 @@ void NetworkManager::authentication(QNetworkReply* reply, QAuthenticator* auth)
     }
 
     // Do not save when private browsing is enabled
-    if (mApp->isPrivateSession()) {
+    if (mApp->isPrivate()) {
         save->setVisible(false);
     }
 
@@ -515,6 +551,10 @@ QNetworkReply* NetworkManager::createRequest(QNetworkAccessManager::Operation op
     QNetworkRequest req = request;
     QNetworkReply* reply = 0;
 
+    if (qzSettings->workOffline) {
+        req.setAttribute(QNetworkRequest::CacheLoadControlAttribute, QNetworkRequest::AlwaysCache);
+    }
+
     // SchemeHandlers
     if (m_schemeHandlers.contains(req.url().scheme())) {
         reply = m_schemeHandlers[req.url().scheme()]->createRequest(op, req, outgoingData);
@@ -539,7 +579,10 @@ QNetworkReply* NetworkManager::createRequest(QNetworkAccessManager::Operation op
         return reply;
     }
 
-    if (req.rawHeader("X-QupZilla-UserLoadAction") == QByteArray("1")) {
+    // Indicates whether the load action was triggered by user (eg. opening new tab, clicking on link, ...)
+    bool userLoadAction = req.rawHeader("X-QupZilla-UserLoadAction") == QByteArray("1");
+
+    if (userLoadAction) {
         req.setRawHeader("X-QupZilla-UserLoadAction", QByteArray());
         req.setAttribute(QNetworkRequest::Attribute(QNetworkRequest::User + 151), QString());
     }
@@ -561,7 +604,7 @@ QNetworkReply* NetworkManager::createRequest(QNetworkAccessManager::Operation op
     //req.setAttribute(QNetworkRequest::HttpPipeliningAllowedAttribute, true);
 
     // Adblock
-    if (op == QNetworkAccessManager::GetOperation) {
+    if (!userLoadAction && op == QNetworkAccessManager::GetOperation) {
         if (!m_adblockManager) {
             m_adblockManager = AdBlockManager::instance();
         }
@@ -584,7 +627,7 @@ void NetworkManager::removeLocalCertificate(const QSslCertificate &cert)
 
     // Delete cert file from profile
     bool deleted = false;
-    QDirIterator it(mApp->currentProfilePath() + "certificates", QDir::Files, QDirIterator::FollowSymlinks | QDirIterator::Subdirectories);
+    QDirIterator it(DataPaths::currentProfilePath() + "/certificates", QDir::Files, QDirIterator::FollowSymlinks | QDirIterator::Subdirectories);
     while (it.hasNext()) {
         const QString filePath = it.next();
         const QList<QSslCertificate> &certs = QSslCertificate::fromPath(filePath);
@@ -618,13 +661,13 @@ void NetworkManager::addLocalCertificate(const QSslCertificate &cert)
     m_localCerts.append(cert);
     QSslSocket::addDefaultCaCertificate(cert);
 
-    QDir dir(mApp->currentProfilePath());
+    QDir dir(DataPaths::currentProfilePath());
     if (!dir.exists("certificates")) {
         dir.mkdir("certificates");
     }
 
     QString certFileName = fileNameForCert(cert);
-    QString fileName = QzTools::ensureUniqueFilename(mApp->currentProfilePath() + "certificates/" + certFileName);
+    QString fileName = QzTools::ensureUniqueFilename(DataPaths::currentProfilePath() + "/certificates/" + certFileName);
 
     QFile file(fileName);
     if (file.open(QFile::WriteOnly)) {
@@ -636,14 +679,39 @@ void NetworkManager::addLocalCertificate(const QSslCertificate &cert)
     }
 }
 
+bool NetworkManager::isIgnoringAllWarnings() const
+{
+    return m_ignoreAllWarnings;
+}
+
 void NetworkManager::setIgnoreAllWarnings(bool state)
 {
     m_ignoreAllWarnings = state;
+    Settings().setValue("SSL-Configuration/IgnoreAllSSLWarnings", m_ignoreAllWarnings);
 }
 
-bool NetworkManager::isIgnoringAllWarnings()
+bool NetworkManager::isDisablingWeakCiphers() const
 {
-    return m_ignoreAllWarnings;
+    return m_disableWeakCiphers;
+}
+
+void NetworkManager::setDisableWeakCiphers(bool state)
+{
+    m_disableWeakCiphers = state;
+    disableWeakCiphers(m_disableWeakCiphers);
+    Settings().setValue("SSL-Configuration/DisableWeakCiphers", m_disableWeakCiphers);
+}
+
+bool NetworkManager::isDisablingSSLv3() const
+{
+    return m_disableSSLv3;
+}
+
+void NetworkManager::setDisableSSLv3(bool state)
+{
+    m_disableSSLv3 = state;
+    disableSSLv3(m_disableSSLv3);
+    Settings().setValue("SSL-Configuration/DisableSSLv3", m_disableSSLv3);
 }
 
 NetworkProxyFactory* NetworkManager::proxyFactory() const
@@ -670,12 +738,14 @@ bool NetworkManager::unregisterSchemeHandler(const QString &scheme, SchemeHandle
     return m_schemeHandlers.remove(scheme) == 1;
 }
 
-void NetworkManager::saveCertificates()
+void NetworkManager::saveSettings()
 {
     Settings settings;
     settings.beginGroup("SSL-Configuration");
     settings.setValue("CACertPaths", m_certPaths);
     settings.setValue("IgnoreAllSSLWarnings", m_ignoreAllWarnings);
+    settings.setValue("DisableWeakCiphers", m_disableWeakCiphers);
+    settings.setValue("DisableSSLv3", m_disableSSLv3);
     settings.endGroup();
 }
 
@@ -685,8 +755,13 @@ void NetworkManager::loadCertificates()
     settings.beginGroup("SSL-Configuration");
     m_certPaths = settings.value("CACertPaths", QStringList()).toStringList();
     m_ignoreAllWarnings = settings.value("IgnoreAllSSLWarnings", false).toBool();
+    m_disableWeakCiphers = settings.value("DisableWeakCiphers", true).toBool();
+    m_disableSSLv3 = settings.value("DisableSSLv3", true).toBool();
     settings.endGroup();
 
+    disableWeakCiphers(m_disableWeakCiphers);
+    disableSSLv3(m_disableSSLv3);
+
     // CA Certificates
     m_caCerts = QSslSocket::defaultCaCertificates();
 
@@ -712,7 +787,7 @@ void NetworkManager::loadCertificates()
     }
     // Local Certificates
 #ifdef Q_OS_WIN
-    QDirIterator it_(mApp->currentProfilePath() + "certificates", QDir::Files, QDirIterator::FollowSymlinks | QDirIterator::Subdirectories);
+    QDirIterator it_(DataPaths::currentProfilePath() + "/certificates", QDir::Files, QDirIterator::FollowSymlinks | QDirIterator::Subdirectories);
     while (it_.hasNext()) {
         QString filePath = it_.next();
         if (!filePath.endsWith(QLatin1String(".crt"))) {
@@ -725,7 +800,7 @@ void NetworkManager::loadCertificates()
         }
     }
 #else
-    m_localCerts = QSslCertificate::fromPath(mApp->currentProfilePath() + "certificates/*.crt", QSsl::Pem, QRegExp::Wildcard);
+    m_localCerts = QSslCertificate::fromPath(DataPaths::currentProfilePath() + "/certificates/*.crt", QSsl::Pem, QRegExp::Wildcard);
 #endif
 
     QSslSocket::setDefaultCaCertificates(m_caCerts + m_localCerts);
@@ -734,8 +809,3 @@ void NetworkManager::loadCertificates()
     new CaBundleUpdater(this, this);
 #endif
 }
-
-void NetworkManager::disconnectObjects()
-{
-    disconnect(this);
-}
