@@ -1,6 +1,6 @@
 /* ============================================================
 * QupZilla - WebKit based browser
-* Copyright (C) 2010-2013  David Rosca <nowrep@gmail.com>
+* Copyright (C) 2010-2014  David Rosca <nowrep@gmail.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -21,7 +21,7 @@
 #include <QSslError>
 #include <QStringList>
 
-#include "qz_namespace.h"
+#include "qzcommon.h"
 #include "networkmanagerproxy.h"
 
 class AdBlockManager;
@@ -29,7 +29,7 @@ class NetworkProxyFactory;
 class QupZillaSchemeHandler;
 class SchemeHandler;
 
-class QT_QUPZILLA_EXPORT NetworkManager : public NetworkManagerProxy
+class QUPZILLA_EXPORT NetworkManager : public NetworkManagerProxy
 {
     Q_OBJECT
 public:
@@ -37,7 +37,7 @@ public:
     QNetworkReply* createRequest(QNetworkAccessManager::Operation op, const QNetworkRequest &request, QIODevice* outgoingData);
 
     void loadSettings();
-    void saveCertificates();
+    void saveSettings();
     void loadCertificates();
 
     QList<QSslCertificate> getCaCertificates() { return m_caCerts; }
@@ -49,16 +49,20 @@ public:
     void setCertificatePaths(const QStringList &paths) { m_certPaths = paths; }
     QStringList certificatePaths() { return m_certPaths; }
 
+    bool isIgnoringAllWarnings() const;
     void setIgnoreAllWarnings(bool state);
-    bool isIgnoringAllWarnings();
+
+    bool isDisablingWeakCiphers() const;
+    void setDisableWeakCiphers(bool state);
+
+    bool isDisablingSSLv3() const;
+    void setDisableSSLv3(bool state);
 
     NetworkProxyFactory* proxyFactory() const;
 
     bool registerSchemeHandler(const QString &scheme, SchemeHandler* handler);
     bool unregisterSchemeHandler(const QString &scheme, SchemeHandler* handler);
 
-    void disconnectObjects();
-
 signals:
     void sslDialogClosed();
 
@@ -70,6 +74,9 @@ private slots:
     void setSSLConfiguration(QNetworkReply* reply);
 
 private:
+    void disableWeakCiphers(bool disable);
+    void disableSSLv3(bool disable);
+
     AdBlockManager* m_adblockManager;
     NetworkProxyFactory* m_proxyFactory;
 
@@ -82,6 +89,8 @@ private:
     QByteArray m_acceptLanguage;
 
     bool m_ignoreAllWarnings;
+    bool m_disableWeakCiphers;
+    bool m_disableSSLv3;
     bool m_doNotTrack;
     bool m_sendReferer;
 };
